import pandas as pd

import lhcsmapi
from lhcsmapi.Time import Time
from lhcsmapi.analysis.busbar.BusbarResistanceQuery import BusbarResistanceQuery
from lhcsmapi.analysis.dfb.DfbQuery import DfbQuery
from lhcsmapi.analysis.pc.PcQuery import PcItQuery
from lhcsmapi.analysis.pic.PicQuery import PicQuery
from lhcsmapi.analysis.qds.QdsQuery import QdsQuery
from lhcsmapi.analysis.qh.QuenchHeaterQuery import QuenchHeaterQuery
from lhcsmapi.analysis.CircuitQuery import CircuitQuery
from lhcsmapi.reference.Reference import Reference


class ItCircuitQuery(PicQuery, PcItQuery, QuenchHeaterQuery, QdsQuery, BusbarResistanceQuery, DfbQuery, CircuitQuery):
    """Class extending the CircuitQuery class with methods for creation of templates of analysis tables in IT circuits.

    """

    def create_report_analysis_template(self, timestamp_fgc: int, init_file_path: str, author='') -> pd.DataFrame:
        """ Method creating a report analysis template for IT circuits

        :param timestamp_fgc: timestamp of an FGC event
        :type timestamp_fgc: int
        :param init_file_path: path to an init file to read LHCSM-HWC notebooks version
        :type init_file_path: str
        :param author: NICE account name of the analysis author
        :type author: str
        :return: pre-filled pd.DataFrame with a template to be completed further with data from an analysis of
            a powering event in IT circuits
        """
        columns = Reference.read_quench_database_columns(circuit_type='IT', table_type='MP3')
        results_table = pd.DataFrame(columns=columns, index=[0])

        results_table['Circuit Name'] = self.circuit_name
        results_table['Circuit Family'] = 'RQX'
        results_table['Period'] = Reference.get_mp3_period(timestamp_fgc)
        results_table['Date (FGC_RQX)'] = Time.to_string_short(timestamp_fgc).split(' ')[0]
        results_table['Time (FGC_RQX)'] = Time.to_string_short(timestamp_fgc).split(' ')[1]
        results_table['Analysis performed by'] = author
        results_table['lhcsmapi version'] = lhcsmapi.__version__
        results_table['lhcsm notebook version'] = lhcsmapi.parse_lhcsmhwc_version(init_file_path)

        return results_table
