import numpy as np


class LinReg(object):

    def __init__(self, theta=None):
        self.theta = theta
        self.sd_error = None

    def fit(self, x, y):
        self.theta = np.linalg.inv(x.T @ x) @ x.T @ y
        return self

    def predict(self, x):
        return x @ self.theta.reshape(-1, 1)

    @staticmethod
    def add_intercept_to_x(x):
        x = x.reshape(-1, 1)
        n = x.shape[0]
        p = x.shape[1] + 1  # plus one because LinearRegression adds an intercept term

        x_with_intercept = np.empty(shape=(n, p), dtype=np.float)
        x_with_intercept[:, 0] = 1
        x_with_intercept[:, 1:p] = x

        return x_with_intercept

    @staticmethod
    def estimate_linear_fit_error(x_intercept, y, y_hat):
        n, p = x_intercept.shape
        residuals = y - y_hat
        residual_sum_of_squares = residuals.T @ residuals
        sigma_squared_hat = residual_sum_of_squares[0, 0] / (n - p)
        var_beta_hat = np.linalg.inv(x_intercept.T @ x_intercept) * sigma_squared_hat

        sd_error = []
        for p_ in range(p):
            sd_error.append(var_beta_hat[p_, p_] ** 0.5)

        return sd_error
