import warnings
from typing import List

import numpy as np

from lhcsmapi.Time import Time
from lhcsmapi.analysis.CircuitAnalysis import CircuitAnalysis


class PicAnalysis(CircuitAnalysis):
    """Base class for power interlock controller (PIC) analysis

    """
    def analyze_pic(self, timestamp_pic: List[int]) -> None:
        """ Method analyzing PIC timestamps. If they differ by more than a ms, then a warning is displayed.

        :param timestamp_pic: list of PIC timestamps
        :return: None
        """
        if any([np.isnan(timestamp_pic_el) for timestamp_pic_el in timestamp_pic]):
            warnings.warn('At least one timestamp is NaN, check of timestamp PIC difference is skipped.', stacklevel=2)
        else:
            dt_pic = abs(timestamp_pic[0] - timestamp_pic[1])
            if dt_pic > 1e6:
                warnings.warn('PIC timestamps (%s) and (%s) differ by more than 1 ms (%d ns)' %
                              (Time.to_string_short(timestamp_pic[0]), Time.to_string_short(timestamp_pic[1]), dt_pic))
            else:
                print('PIC timestamps (%s) and (%s) are less then 1 ms away.' %
                      (Time.to_string_short(timestamp_pic[0]), Time.to_string_short(timestamp_pic[1])))
