import unittest
import warnings
import pandas as pd

from lhcsmapi.analysis.decorators import check_dataframe_empty

warning_text = 'All DataFrames are empty, LEADS voltage plot skipped!'


class TestDecorators(unittest.TestCase):
    @check_dataframe_empty(mode='all', warning=warning_text)
    def analyze_leads_voltage(self, u_df, u_dfs):
        pass

    @check_dataframe_empty(mode='all', warning=warning_text)
    def plot_leads_voltage(self, circuit_name, u_dfs):
        pass

    # analyze_leads_voltage
    def test_analyze_leads_voltage_all_empty(self):

        with warnings.catch_warnings(record=True) as w:
            self.analyze_leads_voltage(pd.DataFrame(), [pd.DataFrame(), pd.DataFrame()])
            self.assertEqual(str(w[0].message), warning_text)

    def test_analyze_leads_voltage_scalar_empty(self):

        with warnings.catch_warnings(record=True) as w:
            self.analyze_leads_voltage(pd.DataFrame(), [pd.DataFrame(index=[0], data=[1], columns=['a']),
                                                        pd.DataFrame(index=[0], data=[1], columns=['a'])])
            self.assertEqual(str(w[0].message), warning_text)

    def test_analyze_leads_voltage_all_non_empty(self):

        with warnings.catch_warnings(record=True) as w:
            self.analyze_leads_voltage(pd.DataFrame(index=[0], data=[1], columns=['a']),
                                                    [pd.DataFrame(index=[0], data=[1], columns=['a']),
                                                     pd.DataFrame(index=[0], data=[1], columns=['a'])])
            self.assertListEqual([], w)

    # plot_leads_voltage
    def test_plot_leads_voltage_all_empty(self):

        with warnings.catch_warnings(record=True) as w:
            self.plot_leads_voltage('RB.A12', [pd.DataFrame(), pd.DataFrame()])
            self.assertEqual(str(w[0].message), warning_text)

    def test_plot_leads_voltage_all_non_empty(self):

        with warnings.catch_warnings(record=True) as w:
            self.plot_leads_voltage('RB.A12', [pd.DataFrame(index=[0], data=[1], columns=['a']),
                                               pd.DataFrame(index=[0], data=[1], columns=['a'])])
            self.assertListEqual([], w)

