
import unittest
import datetime

from lhcsmapi.pyedsl.dbsignal.DbSignal import DbSignal, validate_key_in_kwargs


class TestDbSignal(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_validate_key_in_kwargs_key_not_in_kwargs_returns_none(self):
        """Test if the key passed isn't in kwargs then None is returned"""
        # arrange
        kwargs = {"string": "string", "int": 1, "float": 1.5, "date": datetime.datetime.now()}
        # act
        result = validate_key_in_kwargs("key", str, **kwargs)
        # assert
        self.assertIsNone(result)

    def test_validate_key_in_kwargs_key_is_kwargs_validt_type_right_object_returned(self):
        """Test valid use case. Test that if the  key passed is in kwargs and the result is of the right type
        then this object is returned"""
        kwargs = {"string": "string", "int": 1, "float": 1.5, "date": datetime.datetime.now()}

        testCases = [
            ("string", str),
            ("int", int),
            ('float', float),
            ('date', datetime.datetime) #datetime included to test arbitrary type
        ]
        for key, type in testCases:
            with self.subTest(name=key):
                result = validate_key_in_kwargs(key, type, **kwargs)
                # assert
                self.assertIsInstance(result, type)

    def test_validate_key_in_kwargs_key_is_kwargs_invalid_type_type_error_thrown(self):
        """Test invalid use case. Test that if the key passed is in kwargs  but the result is of the wrong type
                then a TypeError is Thrown"""
        kwargs = {"string": 1, "int": "string", "float": datetime.datetime.now(), "date": 1.5}

        testCases = [
            ("string", str),
            ("int", int),
            ('float', float),
            ('date', datetime.datetime)  # datetime included to test arbitrary type
        ]
        for key, type in testCases:
            with self.subTest(name=key):
                with self.assertRaises(TypeError):
                    validate_key_in_kwargs(key, type, **kwargs)


if __name__ == '__main__':
    unittest.main()
