import unittest
import pandas as pd

from pathlib import Path
import os

from lhcsmapi.pyedsl.ResistanceBuilder import ResistanceBuilder


def read_csv(file_name):
    path = Path(os.path.dirname(__file__))
    full_path = os.path.join(str(path.parent), 'resources/pyedsl/query_builder/{}'.format(file_name))
    return pd.read_csv(full_path, index_col=0)


def read_csv_str(file_name):
    path = Path(os.path.dirname(__file__))
    full_path = os.path.join(str(path.parent), 'resources/pyedsl/query_builder/{}'.format(file_name))

    read_df = pd.read_csv(full_path, index_col=0, dtype={1525982105994000000: object})

    return read_df


class TestResistanceBuilder(unittest.TestCase):

    def test_resistanceBuilder(self):
        # arrange
        spark = None
        t_start_inj = 1526898157236000000
        t_end_sb = 1526903352338000000
        circuit_type = 'RB'
        plateau_translate_udf = None
        signal_type = 'U_MAG'

        mock_i_meas = read_csv('nxcals_fq/RB_I_MEAS_Debug.csv')
        mock_u_mag = read_csv('nxcals_fq/RB_U_MAG_Debug_No_correction.csv')

        ResistanceBuilder().with_busbar_voltage(mock_u_mag) \
            .with_busbar_current(mock_i_meas) \
            .calculate_mean_resistance(circuit_type) \
            .convert_to_row(index=t_start_inj) \
            .df

        # assert
        metadata_ref_i_meas = read_csv('nxcals_fq/RB_I_MEAS_Debug.csv')
        pd.testing.assert_frame_equal(metadata_ref_i_meas, mock_i_meas)

    def test_resistanceBuilder_fill_missing_and_sort(self):
        # arrange
        t_start_inj = 1525982105994000000
        circuit_type = 'RB'

        mock_i_meas = read_csv('nxcals_resistance_builder/RB_I_MEAS_FQ_Resistance_1525982105994000000.csv')
        mock_u_mag = read_csv('nxcals_resistance_builder/RB_U_RES_FQ_Resistance_1525982105994000000.csv')

        mock_res_filled_sorted = read_csv_str('nxcals_resistance_builder/RB_U_MAG_Resistances_Sorted_Filled.csv')

        res_busbar_df = ResistanceBuilder().with_busbar_voltage(mock_u_mag) \
            .with_busbar_current(mock_i_meas) \
            .calculate_mean_resistance(circuit_type) \
            .convert_to_row(index=t_start_inj) \
            .fill_missing_columns(signal_name='R_MAG') \
            .sort_columns_busbar_location(signal_name='R_MAG') \
            .df

        # assert
        mock_res_filled_sorted.index = mock_res_filled_sorted.index.astype(object)
        mock_res_filled_sorted.columns.name = 'nxcals_variable_name'

        for col in mock_res_filled_sorted:
            mock_res_filled_sorted[col] = mock_res_filled_sorted[col].astype(object)

        pd.testing.assert_frame_equal(mock_res_filled_sorted, res_busbar_df)
